unit MessageDigestDriver;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TfrmMessageDigestDriver = class(TForm)
    cmdTransform: TButton;
    rgAlgorithm: TRadioGroup;
    cmdClose: TButton;
    gbHashValue: TGroupBox;
    txtHashValue: TEdit;
    gbPerformance: TGroupBox;
    lblBytesProcessedLabel: TLabel;
    lblBytesProcessed: TLabel;
    lblTimeLabel: TLabel;
    lblTime: TLabel;
    lblBytesPerSecondLabel: TLabel;
    lblBytesPerSecond: TLabel;
    gbSourceData: TGroupBox;
    txtSource: TEdit;
    lblInputFrom: TLabel;
    rdoSourceFile: TRadioButton;
    rdoSourceString: TRadioButton;
    lblSource: TLabel;
    txtIterations: TEdit;
    udIterations: TUpDown;
    lblIterations: TLabel;
    gbAuthentication: TGroupBox;
    rdoAuthenticationNone: TRadioButton;
    rdoAuthenticationHMAC: TRadioButton;
    txtKey: TEdit;
    lblKey: TLabel;
    cmdBrowse: TButton;
    OpenDialog: TOpenDialog;
    procedure cmdTransformClick(Sender: TObject);
    procedure cmdCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure txtIterationsKeyPress(Sender: TObject; var Key: Char);
    procedure rdoSourceStringClick(Sender: TObject);
    procedure rdoAuthenticationNoneClick(Sender: TObject);
    procedure cmdBrowseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    T1, T2: DWORD;
    BytesDigested: Longint;
    procedure ClearLabels;
    procedure ComputeHashValue;
    procedure UseNoAuthentication(const N: Integer);
    procedure UseHMACAuthentication(const N: Integer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMessageDigestDriver: TfrmMessageDigestDriver;

implementation

uses MessageDigests, MessageAuthenticationCodes;

{$R *.DFM}

const
  RadioToMDClass: array [0..4] of TMessageDigestClass = (
   TMD2, TMD4, TMD5, TSHA1, TRIPEMD160
  );

procedure TfrmMessageDigestDriver.UseNoAuthentication(const N: Integer);
var
  MD: TMessageDigest;
  S: string;
  Stream: TFileStream;
  I: Longint;
begin
  MD := RadioToMDClass[N].Create;
  S := txtSource.Text;
  if rdoSourceFile.Checked then begin
    Stream := TFileStream.Create(S, fmOpenRead);
    T1 := GetTickCount;
    for I := 1 to StrToInt(txtIterations.Text) do begin
      Stream.Seek(0, soFromBeginning);
      MD.TransformStream(Stream);
    end;
    MD.Complete;
    T2 := GetTickCount;
    Stream.Free;
  end
  else if rdoSourceString.Checked then begin
    T1 := GetTickCount;
    for I := 1 to StrToInt(txtIterations.Text) do begin
      MD.TransformString(S);
    end;
    MD.Complete;
    T2 := GetTickCount;
  end
  else begin
    MessageDlg('Please select a source.', mtInformation, [mbOK], 0);
    T1 := 0;
    T2 := 0;
  end;
  BytesDigested := MD.NumBytesDigested;
  txtHashValue.Text := MD.HashValue;
  MD.Free;
end;

procedure TfrmMessageDigestDriver.UseHMACAuthentication(const N: Integer);
var
  HMAC: THMAC;
  Key: string;
  S: string;
  Stream: TFileStream;
  I: Longint;
begin
  Key := txtKey.Text;
  HMAC := THMAC.Create(RadioToMDClass[N], Key[1], Length(Key), 0);
  S := txtSource.Text;
  if rdoSourceFile.Checked then begin
    Stream := TFileStream.Create(S, fmOpenRead);
    T1 := GetTickCount;
    for I := 1 to StrToInt(txtIterations.Text) do begin
      Stream.Seek(0, soFromBeginning);
      HMAC.TransformStream(Stream);
    end;
    HMAC.Complete;
    T2 := GetTickCount;
    Stream.Free;
  end
  else if rdoSourceString.Checked then begin
    T1 := GetTickCount;
    for I := 1 to StrToInt(txtIterations.Text) do begin
      HMAC.TransformString(S);
    end;
    HMAC.Complete;
    T2 := GetTickCount;
  end
  else begin
    MessageDlg('Please select a source.', mtInformation, [mbOK], 0);
    T1 := 0;
    T2 := 0;
  end;
  BytesDigested := HMAC.NumBytesDigested;
  txtHashValue.Text := HMAC.HashValue;
  HMAC.Free;
end;

procedure TfrmMessageDigestDriver.ComputeHashValue;
const
  srcFile = 0;
  srcString = 1;
var
  MSecs: DWORD;
  Secs: Double;
  N: Integer;
begin
  ClearLabels;
  N := rgAlgorithm.ItemIndex;
  if (N < Low(RadioToMDClass)) or (N > High(RadioToMDClass)) then begin
    MessageDlg('Please select a message digest algorithm.', mtInformation,
               [mbOK], 0);
    Exit;
  end;
  if rdoAuthenticationNone.Checked then
    UseNoAuthentication(N)
  else if rdoAuthenticationHMAC.Checked then
    UseHMACAuthentication(N);
  lblBytesProcessed.Caption := IntToStr(BytesDigested);
  MSecs := T2 - T1;
  if MSecs > 0 then begin
    lblTime.Caption := IntToStr(MSecs) + ' ms';
    Secs := MSecs / 1000;
    lblBytesPerSecond.Caption := FormatFloat('.00', BytesDigested / Secs);
  end;
end;

procedure TfrmMessageDigestDriver.cmdTransformClick(Sender: TObject);
const
  Computing = 'Computing...';
begin
  Screen.Cursor := crHourGlass;
  txtHashValue.Text := Computing;
  txtHashValue.Update;
  try
    ComputeHashValue;
  finally
    Screen.Cursor := crDefault;
    if CompareStr(txtHashValue.Text, Computing) = 0 then txtHashValue.Text := '';
  end;
end;

procedure TfrmMessageDigestDriver.cmdCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMessageDigestDriver.FormCreate(Sender: TObject);
begin
  ClearLabels;
  txtHashValue.Text := '';
  rdoSourceFile.OnClick(Self);
  rdoAuthenticationNone.OnClick(Self);
end;

procedure TfrmMessageDigestDriver.ClearLabels;
begin
  lblBytesProcessed.Caption := '0';
  lblTime.Caption := '0 ms';
  lblBytesPerSecond.Caption := '0';
  lblBytesProcessed.Update;
  lblTime.Update;
  lblBytesPerSecond.Update;
end;

procedure TfrmMessageDigestDriver.txtIterationsKeyPress(Sender: TObject;
  var Key: Char);
const
  BackSpace = #8;
begin
  if not (Key in [BackSpace, '0'..'9']) then Key := #0;
end;

procedure TfrmMessageDigestDriver.rdoSourceStringClick(Sender: TObject);
begin
  if rdoSourceFile.Checked then
    lblSource.Caption := 'File name:'
  else if rdoSourceString.Checked then begin
    lblSource.Caption := 'Text:';
  end;
  cmdBrowse.Enabled := rdoSourceFile.Checked
end;

procedure TfrmMessageDigestDriver.rdoAuthenticationNoneClick(
  Sender: TObject);
begin
  lblKey.Enabled := not rdoAuthenticationNone.Checked;
  txtKey.Enabled := lblKey.Enabled;
end;

procedure TfrmMessageDigestDriver.cmdBrowseClick(Sender: TObject);
begin
  if OpenDialog.Execute then txtSource.Text := OpenDialog.FileName;
end;

procedure TfrmMessageDigestDriver.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

end.
